from api import Api
from azure.storage.file import FileService
import logging
import os
from netifaces import interfaces, ifaddresses, AF_INET
import socket
from json import load
from urllib2 import urlopen
from gpuinfo.nvidia import get_gpus
import click


def init_environment():
    os.system('curl https://public-packages.oss-cn-beijing.aliyuncs.com/install_nvidia_docker.sh|sh -')


def init_file_server_by_key(key):
    config = Api(key=key).get_config_by_key().get('data')

    file_service = FileService(account_name=config.get('account_name'),
                               account_key=config.get('account_key'),
                               endpoint_suffix=config.get('endpoint_suffix'))

    logging.basicConfig(format='%(asctime)s %(name)-20s %(levelname)-5s %(message)s', level=logging.INFO)
    logger = logging.getLogger('azure.storage')
    handler = logging.StreamHandler()
    formatter = logging.Formatter('%(asctime)s %(name)-20s %(levelname)-5s %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(logging.INFO)

    return file_service


def get_folder_name_from_path(path):
    return os.path.basename(path)


def listdir(path, list_name):
    for file in os.listdir(path):
        file_path = os.path.join(path, file)
        if os.path.isdir(file_path):
            listdir(file_path, list_name)
        else:
            list_name.append(file_path)


def ip4_addresses():
    ip_list = []
    for interface in interfaces():
        for link in ifaddresses(interface).get(AF_INET, ()):
            ip_list.append(link['addr'])
    return ip_list


def get_host_ip():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip


def get_public_ip():
    return load(urlopen('http://jsonip.com'))['ip']


def is_empty(key):
    return key == '' or key is None


def get_device_info():
    gpus = []
    try:
        for gpu in get_gpus():
            gpus.append({
                "Name": gpu.get('name'),
                "Memory": gpu.get('total_memory')
            })
    except:
        click.echo("get gpu information error ")
    return {
        "hostname": socket.gethostname(),
        "ip_address": get_host_ip(),
        "gpu": gpus,
        "public_ip": get_public_ip()
    }



import click
import get as get_package
import put as put_package
import watch as watch_package
import utils
from os import system
from click import echo
from config import VERSION
from container import Container


@click.group()
def get_group():
    pass


@get_group.command()
@click.argument('key', required=True)
@click.argument('path', required=True)
@click.option('--code/--no-code', default=True, help='get code source')
@click.option('--result/--no-result', default=True, help='get result')
def get(key, path, code, result):
    if code:
        get_package.Get(key).get(path)
    if result:
        get_package.Get(key, 'result').get(path)


@click.group()
def put_group():
    pass


@put_group.command()
@click.argument('key', required=True)
@click.argument('path', required=True)
@click.option('--folder/--no-folder', default=True, help='upload directory')
@click.option('--code/--no-code', default=False, help='upload code')
@click.option('--result/--no-result', default=False, help='put result')
def put(key, path, folder, code, result):
    if code:
        put_package.Put(key).put(path, folder)
    if result:
        put_package.Put(key, 'result').put(path, folder)


@click.group()
def watch_group():
    pass


@watch_group.command()
@click.option('-t', '--time', default=10, help='loop time (s)')
def watch(time):
    watch_package.Watch(time).watch()


@click.group()
def init_group():
    pass


@init_group.command()
def init():
    utils.init_environment()


@click.group()
def upgrade_group():
    pass


@upgrade_group.command()
def upgrade():
    system('pip install --upgrade eubh --no-cache-dir')


@click.group()
def version_group():
    pass


@version_group.command()
def version():
    echo(VERSION)


@click.group()
def container_group():
    pass


@container_group.command()
@click.argument('key', required=True)
@click.option('--lists/--no-lists', default=False, help='list container name')
@click.option('--log', help='Get the container log')
def container(key, lists, log):
    if lists:
        Container(key).lists()
    elif log:
        Container(key).log(log, is_watch=True)


cli = click.CommandCollection(
    sources=[get_group, put_group, watch_group, init_group, upgrade_group, version_group, container_group])


def main():
    cli()


if __name__ == "__main__":
    cli()



import os
import threading
from uuid import getnode as get_mac

import docker
from apscheduler.schedulers.background import BlockingScheduler
from click import echo

import api
import get
import put
import status
import utils


def clean_file():
    echo('remove /root/code and /root/result')
    os.popen('rm -rf /root/code result')
    echo('remove files successful.')


def create_directories():
    os.makedirs('/root/code')
    os.makedirs('/root/result')


# def cat_logs_in_background(container):
#     container_id = container.id
#     for line in container.logs(stream=True):
#         with open('/root/result/%s.log' % container_id, 'a') as f:
#             f.writelines(line)
#             f.close()


class CatContainerLog(threading.Thread):
    def __init__(self, container, watch):
        threading.Thread.__init__(self)
        self.container = container
        self.watch = watch

    def run(self):
        try:
            container_id = self.container.id
            for line in self.container.logs(stream=True, tail=3):
                with open('/root/result/%s.log' % container_id, 'a') as f:
                    self.watch.api.store_project_container_log({'container_id': container_id, 'text': line.strip()})
                    f.writelines(line)
                    f.close()
        except:
            echo("container logs release")


class Watch:
    def __init__(self, time):
        self.time = time
        self.key = None
        self.api = api.Api()
        self.docker_client = docker.from_env()
        self.can_init = True

    def clean_docker(self):
        client = self.docker_client
        for container in client.containers.list():
            container.stop()

    def run_cmd_and_upload_result(self, pivot):
        cmd = pivot.get('cmd')
        if not utils.is_empty(cmd):
            cmd_result_out_put = os.popen(cmd)
            cmd_result_out_put_string = cmd_result_out_put.read()
            self.api.project_machine_update_cmd({"id": pivot.get('id'), "result": cmd_result_out_put_string})
            cmd_result_out_put.close()

    def upload_machine_information(self):
        client = self.docker_client
        current_status = status.Status.IDLE.value

        client_containers_count = len(client.containers.list())
        if client_containers_count == 0:
            if not utils.is_empty(self.key):
                current_status = status.Status.COMPLETE.value
                # put.Put(self.key, 'result').put('/root/result', True)
        else:
            current_status = status.Status.IDLE.value

        if client_containers_count > 0:
            if utils.is_empty(self.key):
                self.clean_docker()
            current_status = status.Status.RUNNING.value

        data = utils.get_device_info()
        data['status_info'] = {
            'key': self.key,
            'status': current_status
        }
        data['mac'] = "machine_%s" % get_mac()
        response = self.api.upload_machine_and_get_task(data)

        print(data, response)

        if type(response) is not list:
            option = response.get('option')
            project = response.get('project')
            pivot = response.get('pivot')

            key = None
            if project is not None:
                key = project.get('key')

            if not utils.is_empty(key):
                if option == 'init' and self.can_init:
                    self.can_init = False
                    clean_file()
                    create_directories()
                    get.Get(key).get('/root')
                    self.key = key
                    self.run_cmd_and_upload_result(pivot)
                    for container in client.containers.list():
                        # start_new_thread(cat_logs_in_background(container))
                        cat_logs_background_threading = CatContainerLog(container, self)
                        cat_logs_background_threading.start()
                        # wait till the background thread is done
                        # cat_logs_background_threading.join()

                elif option == 'cmd':
                    self.run_cmd_and_upload_result(pivot)

            if option == 'clean':
                self.clean_docker()
                put.Put(self.key, 'result').put('/root/result', True)
                self.key = ''
                self.can_init = True

    def watch(self):
        scheduler = BlockingScheduler()
        scheduler.add_job(self.upload_machine_information, 'interval', seconds=self.time, max_instances=2)
        try:
            scheduler.start()
        except (KeyboardInterrupt, SystemExit):
            scheduler.shutdown()
